@@ -32,7 +32,9 @@ WARC::Index::Entry - abstract base class for WARC::Index entries
 
 =head1 DESCRIPTION
 
-=head2 Methods
+=head2 Common Methods
+
+Entries from all index systems support these methods:
 
 =over
 
@@ -50,7 +52,8 @@ the record does not match the criteria at all.  An undefined value
 indicates that the entry is from an index that does not store the
 information needed to evaluate distance for that search key.  Undefined
 values are ignored when computing the summarized distance, but the
-summarized distance will be negative if any keys do not match at all.
+summarized distance will be negative if any keys do not match at all and
+itself undefined if none of the requested keys can be evaluated.
 
 For details on available search keys, see the L<"Search Keys"
 section|WARC::Collection/"Search Keys"> of the C<WARC::Collection> page.
@@ -63,6 +66,8 @@ sub distance {
   unless (defined wantarray)
     { carp "calling 'distance' method in void context"; return }
 
+  croak "no arguments given to 'distance' method"
+    unless scalar @_;
   croak "odd number of arguments given to 'distance' method"
     if scalar @_ % 2;
 
@@ -100,8 +105,10 @@ sub _distance_summary {
 }
 
 our %_distance_value_map =
-  ( time => 'time',
-    url => 'url', url_prefix => 'url',
+  ( time	=> [numeric => 'time'],
+    record_id	=> [exact => 'record_id'],
+    url		=> [exact => 'url'],
+    url_prefix	=> [prefix => 'url'],
   );
 
 sub _distance_for_item {
@@ -111,19 +118,21 @@ sub _distance_for_item {
 
   croak "index distance requested for unknown item $item"
     unless defined $_distance_value_map{$item};
-  my $actual = $self->value($_distance_value_map{$item});
+  my $actual = $self->value($_distance_value_map{$item}[1]);
 
   return undef unless defined $actual;
 
+  my $mode = $_distance_value_map{$item}[0];
+
   # uncoverable branch false count:3
-  if ($item eq 'time') {
+  if ($mode eq 'exact') {
+    return -(0+($sought ne $actual));
+  } elsif ($mode eq 'numeric') {
     return abs($actual - $sought);
-  } elsif ($item eq 'url_prefix') {
+  } elsif ($mode eq 'prefix') {
     return -1 unless $sought eq substr $actual, 0, length $sought;
     return length($actual) - length($sought);
-  } elsif ($item eq 'url') {
-    return -(0+($sought ne $actual));
-  } else { die "missing branch for item $item" } # uncoverable statement
+  } else { die "unknown mode '$mode' for item '$item'" } # uncoverable statement
 }
 
 =item $index = $entry-E<gt>index
@@ -200,6 +209,26 @@ sub tag {
 
 =back
 
+=head2 Optional Methods
+
+Some index entries may additionally support any of these methods:
+
+=over
+
+=item $next_entry = $entry-E<gt>next
+
+Indexes with an inherent sequence of entries may provide a method to obtain
+the next entry in the index.  Some index systems have this, while others do
+not have a meaningful order amongst their entries.
+
+=item $position = $entry-E<gt>entry_position
+
+Indexes with an inherent sequence of entries may provide a method to obtain
+some kind of index-specific entry number or location parameter.  This is
+most useful for metaindexes to record the location of an index entry.
+
+=back
+
 =cut
 
 1;
